# coding=utf-8
#drapi- Draugiem.lv API wrapperis priekš Python
#by Reinis Veips, http://wot.lv

import urllib
from lxml import etree

class APIError:
    """Exception klase drapi klasei. Kļūdu kodu skaidrojumus skatīt draugiem.lv API dokumentācijā"""
    code=None #Kļūdas kods
    description = None #Kļūdas apraksts
    def __init__(self,error_code,error_description):
        self.code=error_code
        self.description=error_description
    
    def __str__(self):
        return self.code + ': '+self.description


class drapi:
    """Draugiem.lv API wrapperis. Visas get_* f-jas atgriež Python dictionary'us,
    kuru struktūra ir ļoti līdzīga XML struktūrai, taču sīkāk jāskatās paraugā"""
    API_URL = 'http://api.draugiem.lv/xml/'
    authcode = None
    appkey = None
    userkey = None
    def __init__(self,appkey,userkey = None):
        """Klases konstruktors.
        appkey- aplikācijas atslēga, userkey- lietotāja atslēga, var norādīt arī vēlāk"""
        self.appkey = appkey
        self.userkey = userkey
        
    def __query(self,params):
        """Shorthand's datu pieprasījumam un atbildes atgriešanai.
        Ja nepieciešams, izmet exception'u kļūdas gadījumā."""
        resp=urllib.urlopen(self.API_URL,urllib.urlencode(params))
        resp_tree = etree.parse(resp).getroot()
        error = resp_tree.find('error')
        if (etree.iselement(error)):
            raise APIError(error.get('code'),error.text)
        return resp_tree
                
    def __dictionarize_users(self,users_element):
        users={}
        for user in users_element:
            id=int(user.get('uid'))
            if not id in users:
                users[id]={}
                for elem in user:
                    users[id][elem.tag]=elem.text
                users[id]['id']=id
        return users
    
    def __dictionarize_messages(self,messages_element,users):
        messages = {}
        for message in messages_element:
            id=int(message.get('id'))
            messages[id]={}
            for elem in message:
                if elem.tag=='uid':
                    messages[id]['user']=users[int(elem.text)]
                else:
                    messages[id][elem.tag]=elem.text
        return messages
            
    def start_authorization(self,email):
        """Aplikācijas autorizācijas pieprasījums.
        Pēc tam, kad lietotājs apstiprinājis aplikāciju,jāizsauc end_authorization()
        Atgriež authcode vērtību, kas principā klases lietotājam nav svarīgi, taču var noderēt,
        ja autorizāciju pabeidz cita instance."""
        resp = self.__query({'action':'authorize','app':self.appkey,'email':email})
        self.authcode = resp.find('authcode').text
        return self.authcode
        
    def finish_authorization(self,authcode=None):
        """Autorizācijas pabeigšana: jāizsauc 20 min laikā kopš lietotājs ir apstiprinājis aplikāciju."""
        if (authcode != None) or (self.authcode!=None):
            if self.authcode== None:
                self.authcode=authcode
            resp = self.__query({'action':'authorize','app':self.appkey,'code':self.authcode})
            self.userkey = resp.find('apikey').text
            
            return self.userkey
    
    def get_user_data(self,userkey=None):
        """Savāc lietotāja datus ('login' procedūra, kā to sauc API autori)."""
        if userkey!=None:
            self.userkey=userkey
        resp=self.__query({'action':'login','app':self.appkey,'apikey':self.userkey})
        ret={}
        for elem in resp.find('login'):
            ret[elem.tag]=elem.text
        return ret
    
    def get_counters(self,userkey=None):
        """Savāc lietotāja counter'us."""
        if userkey!=None:
            self.userkey=userkey
        resp=self.__query({'action':'counters','app':self.appkey,'apikey':self.userkey})
        ret={}
        for elem in resp.find('counters'):
            ret[elem.tag]=elem.text
       
        return ret
    
    def get_activities(self,userkey=None,show='friends',types=None,timestamp=None):
        """Dabū aktivitāties no draugu vai paša profila."""
        if userkey!=None:
            self.userkey=userkey
        params={'app':self.appkey,'apikey':self.userkey,'action':'activities','show':show}
        if types!=None:
            params['types']=types
        if timestamp!=None:
            params['timestamp']=timestamp
        resp=self.__query(params)
        users=self.__dictionarize_users(resp.find('users'))
        events={}
        i=0
        for event in resp.find('events'):
            events[i]={}
            for elem in event:
                if elem.tag=='uid':
                    events[i]['user']=users[int(elem.text)]
                else:
                    events[i][elem.tag]=elem.text
            i=i+1
            
        
        return events
        
    def get_messages(self,userkey=None,only_unread=False):
        """Savāc pēdējās ziņas no lietotāja konta. ja only_unread ir True, tad atgriež tikai nelasītās ziņas."""
        if userkey!=None:
            self.userkey=userkey
        params={'app':self.appkey,'apikey':self.userkey,'action':'messages'}
        if only_unread:
            params['show']='unread'
        resp= self.__query(params)
        users=self.__dictionarize_users(resp.find('users'))
        messages = self.__dictionarize_messages(resp.find('messages'),users)
        
        return messages
        
        
        
        
    
        
